/*
 * Copyright 2016 - 2022 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include "nostalgia/studio/lib/undostack.hpp"
#include <nostalgia/core/gfx.hpp>
#include <nostalgia/studio/studio.hpp>
#include <ox/std/memory.hpp>

namespace nostalgia::core {

class PaletteEditorImGui: public studio::Editor {

	friend class AddColorCommand;
	friend class RemoveColorCommand;
	friend class UpdateColorCommand;
	friend class MoveColorCommand;

	private:
		Context *m_ctx = nullptr;
		ox::String m_itemName;
		ox::String m_itemPath;
		Palette m_pal;
		std::size_t m_selectedRow = 0;
		studio::UndoStack m_undoStack;

		PaletteEditorImGui() noexcept = default;

	public:
		static ox::Result<PaletteEditorImGui*> make(Context *ctx, const ox::String &path) noexcept;

		/**
		 * Returns the name of item being edited.
		 */
		const ox::String &itemName() const override;

		void draw(core::Context*) noexcept override;

		studio::UndoStack *undoStack() noexcept final;

	protected:
		void addColor(int idx, Color16 c) noexcept;

		void rmColor(int idx) noexcept;

		void updateColor(int idx, Color16) noexcept;

		void moveColor(int idx, int offset) noexcept;

		void saveItem() override;

	private:
		ox::Error markUnsavedChanges(int) noexcept;

};

}
