/*
 * Copyright 2016 - 2022 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include "undostack.hpp"

namespace nostalgia::studio {

bool UndoCommand::mergeWith(const UndoCommand*) noexcept {
	return false;
}

void UndoStack::push(UndoCommand *cmd) noexcept {
	for (const auto i = m_stackIdx; i < m_stack.size();) {
		oxIgnoreError(m_stack.erase(i));
	}
	auto cmdId = cmd->commandId();
	cmd->redo();
	redoTriggered.emit(cmdId);
	changeTriggered.emit(cmdId);
	if (!m_stack.size() || !m_stack.back().value->mergeWith(cmd)) {
		m_stack.emplace_back(cmd);
		++m_stackIdx;
	} else {
		delete cmd;
	}
}

void UndoStack::redo() noexcept {
	if (m_stackIdx < m_stack.size()) {
		auto &c = m_stack[m_stackIdx++];
		auto cmdId = c->commandId();
		c->redo();
		redoTriggered.emit(cmdId);
		changeTriggered.emit(cmdId);
	}
}

void UndoStack::undo() noexcept {
	if (m_stackIdx) {
		auto &c = m_stack[--m_stackIdx];
		auto cmdId = c->commandId();
		c->undo();
		undoTriggered.emit(cmdId);
		changeTriggered.emit(cmdId);
	}
}

}
